@@ -1,11 +1,13 @@
+from abc import abstractmethod
 from dataclasses import dataclass, field
-from datetime import datetime, timezone
+from datetime import datetime
 from ipaddress import ip_interface, IPv6Interface
-from typing import List, Optional, Any, Dict, Sequence, Callable
+from typing import List, Optional, Any, Dict, Sequence, TypeVar, Generic
 
 from annet.annlib.netdev.views.dump import DumpableView
-from annet.annlib.netdev.views.hardware import HardwareView, lag_name, svi_name
+from annet.annlib.netdev.views.hardware import HardwareView, lag_name
 from annet.storage import Storage
+from annet.vendors import registry_connector
 
 
 @dataclass
@@ -18,6 +20,11 @@ class Entity(DumpableView):
         return self.name
 
 
+@dataclass
+class EntityWithSlug(Entity):
+    slug: str
+
+
 @dataclass
 class Label:
     value: str
@@ -42,7 +49,6 @@ class DeviceIp(DumpableView):
     id: int
     display: str
     address: str
-    family: int
 
     @property
     def _dump__list_key(self):
@@ -53,7 +59,7 @@ class DeviceIp(DumpableView):
 class Prefix(DumpableView):
     id: int
     prefix: str
-    site: Optional[Entity]
+    # `site` deprecated since v4.2, replace in derived classes.
     vrf: Optional[Entity]
     tenant: Optional[Entity]
     vlan: Optional[Entity]
@@ -70,18 +76,21 @@ class Prefix(DumpableView):
         return self.prefix
 
 
+_PrefixT = TypeVar("_PrefixT", bound=Prefix)
+
+
 @dataclass
-class IpAddress(DumpableView):
+class IpAddress(DumpableView, Generic[_PrefixT]):
     id: int
-    assigned_object_id: int
+    assigned_object_id: int | None
     display: str
     family: IpFamily
     address: str
     status: Label
-    tags: List[Entity]
+    tags: List[EntityWithSlug]
     created: datetime
     last_updated: datetime
-    prefix: Optional[Prefix] = None
+    prefix: Optional[_PrefixT] = None
     vrf: Optional[Entity] = None
 
     @property
@@ -91,7 +100,7 @@ class IpAddress(DumpableView):
 
 @dataclass
 class InterfaceConnectedEndpoint(Entity):
-    device: Entity
+    device: Entity | None
 
 
 @dataclass
@@ -111,8 +120,59 @@ class InterfaceVlan(Entity):
     vid: int
 
 
+def vrf_object(vrf: str | None) -> Entity | None:
+    if vrf is None:
+        return None
+    else:
+        return Entity(id=0, name=vrf)
+
+
+_IpAddressT = TypeVar("_IpAddressT", bound=IpAddress)
+
+
+_DeviceIPT = TypeVar("_DeviceIPT", bound=DeviceIp)
+
+
 @dataclass
-class Interface(Entity):
+class FHRPGroup(Generic[_DeviceIPT]):
+    id: int
+    group_id: int
+    display: str
+    protocol: str
+    description: str | None
+
+    name: str
+    auth_type: str | None
+    auth_key: str
+    tags: list[EntityWithSlug]
+    custom_fields: dict[str, Any]
+    ip_addresses: list[_DeviceIPT]
+    comments: str | None = None
+
+
+_FHRPGroupT = TypeVar("_FHRPGroupT", bound=FHRPGroup)
+
+
+@dataclass
+class FHRPGroupAssignment(Generic[_FHRPGroupT]):
+    id: int
+    display: str
+    priority: int
+    group: _FHRPGroupT
+    fhrp_group_id: int
+
+    interface_type: str | None
+    interface_id: int | None
+
+
+_FHRPGroupAssignmentT = TypeVar(
+    "_FHRPGroupAssignmentT",
+    bound=FHRPGroupAssignment,
+)
+
+
+@dataclass
+class Interface(Entity, Generic[_IpAddressT, _FHRPGroupAssignmentT]):
     device: Entity
     enabled: bool
     description: str
@@ -121,56 +181,61 @@ class Interface(Entity):
     mode: Optional[InterfaceMode]
     untagged_vlan: Optional[InterfaceVlan]
     tagged_vlans: Optional[List[InterfaceVlan]]
+    tags: List[EntityWithSlug] = field(default_factory=list)
     display: str = ""
-    ip_addresses: List[IpAddress] = field(default_factory=list)
     vrf: Optional[Entity] = None
     mtu: int | None = None
     lag: Entity | None = None
     lag_min_links: int | None = None
+    speed: int | None = None
+
+    ip_addresses: List[_IpAddressT] = field(default_factory=list)
+    count_ipaddresses: int = 0
+    fhrp_groups: List[_FHRPGroupAssignmentT] = field(default_factory=list)
+    count_fhrp_groups: int = 0
 
     def add_addr(self, address_mask: str, vrf: str | None) -> None:
         addr = ip_interface(address_mask)
-        if vrf is None:
-            vrf_obj = None
-        else:
-            vrf_obj = Entity(id=0, name=vrf)
 
+        # when comparing ip addressess
+        # we dont care about the vrf of an addr
+        # because the actual vrf will be
+        # determined by an interface's vrf
+        # and there can be only one
+        #
+        # also we dont care about a mask
+        # because no device will allow
+        # the same addr with multiple masks
+        for existing_addr in self.ip_addresses:
+            existing = ip_interface(existing_addr.address)
+            if existing.ip == addr.ip:
+                return
+
+        vrf_obj = vrf_object(vrf)
         if isinstance(addr, IPv6Interface):
             family = IpFamily(value=6, label="IPv6")
         else:
             family = IpFamily(value=4, label="IPv4")
+        self._add_new_addr(address_mask, vrf_obj, family)
 
-        for existing_addr in self.ip_addresses:
-            if existing_addr.address == address_mask and (
-                (existing_addr.vrf is None and vrf is None) or
-                (existing_addr.vrf is not None and existing_addr.vrf.name == vrf)
-            ):
-                return
-        self.ip_addresses.append(IpAddress(
-            id=0,
-            display=address_mask,
-            address=address_mask,
-            vrf=vrf_obj,
-            prefix=None,
-            family=family,
-            created=datetime.now(timezone.utc),
-            last_updated=datetime.now(timezone.utc),
-            tags=[],
-            status=Label(value="active", label="Active"),
-            assigned_object_id=self.id,
-        ))
+    @abstractmethod
+    def _add_new_addr(self, address_mask: str, vrf: Entity | None, family: IpFamily):
+        raise NotImplementedError
+
+
+_InterfaceT = TypeVar("_InterfaceT", bound=Interface)
 
 
 @dataclass
-class NetboxDevice(Entity):
+class NetboxDevice(Entity, Generic[_InterfaceT, _DeviceIPT]):
     url: str
     storage: Storage
 
     display: str
     device_type: DeviceType
-    device_role: Entity
+    # `device_role` deprecated since v4.0, replace in derived classes.
     tenant: Optional[Entity]
-    platform: Optional[Entity]
+    platform: Optional[EntityWithSlug]
     serial: str
     asset_tag: Optional[str]
     site: Entity
@@ -178,20 +243,23 @@ class NetboxDevice(Entity):
     position: Optional[float]
     face: Optional[Label]
     status: Label
-    primary_ip: Optional[DeviceIp]
-    primary_ip4: Optional[DeviceIp]
-    primary_ip6: Optional[DeviceIp]
-    tags: List[Entity]
+    primary_ip: Optional[_DeviceIPT]
+    primary_ip4: Optional[_DeviceIPT]
+    primary_ip6: Optional[_DeviceIPT]
+    tags: List[EntityWithSlug]
     custom_fields: Dict[str, Any]
     created: datetime
     last_updated: datetime
+    cluster: Optional[Entity]
+    config_context: Optional[dict[str, Any]]
+    config_template: Optional[dict[str, Any]]
 
     fqdn: str
     hostname: str
     hw: Optional[HardwareView]
     breed: str
 
-    interfaces: List[Interface]
+    interfaces: List[_InterfaceT]
 
     @property
     def neighbors(self) -> List["Entity"]:
@@ -222,27 +290,14 @@ class NetboxDevice(Entity):
         custom_breed_pc = ("Mellanox", "NVIDIA", "Moxa", "Nebius")
         return self.device_type.manufacturer.name in custom_breed_pc or self.breed == "pc"
 
-    def _make_interface(self, name: str, type: InterfaceType) -> Interface:
-        return Interface(
-            name=name,
-            device=self,
-            enabled=True,
-            description="",
-            type=type,
-            id=0,
-            vrf=None,
-            display=name,
-            untagged_vlan=None,
-            tagged_vlans=[],
-            ip_addresses=[],
-            connected_endpoints=[],
-            mode=None,
-        )
+    @abstractmethod
+    def _make_interface(self, name: str, type: InterfaceType) -> _InterfaceT:
+        raise NotImplementedError
 
     def _lag_name(self, lag: int) -> str:
         return lag_name(self.hw, lag)
 
-    def make_lag(self, lag: int, ports: Sequence[str], lag_min_links: int | None) -> Interface:
+    def make_lag(self, lag: int, ports: Sequence[str], lag_min_links: int | None) -> _InterfaceT:
         new_name = self._lag_name(lag)
         for target_interface in self.interfaces:
             if target_interface.name == new_name:
@@ -259,9 +314,9 @@ class NetboxDevice(Entity):
         return lag_interface
 
     def _svi_name(self, svi: int) -> str:
-        return svi_name(self.hw, svi)
+        return registry_connector.get().match(self.hw).svi_name(svi)
 
-    def add_svi(self, svi: int) -> Interface:
+    def add_svi(self, svi: int) -> _InterfaceT:
         name = self._svi_name(svi)
         for interface in self.interfaces:
             if interface.name == name:
@@ -276,7 +331,7 @@ class NetboxDevice(Entity):
     def _subif_name(self, interface: str, subif: int) -> str:
         return f"{interface}.{subif}"
 
-    def add_subif(self, interface: str, subif: int) -> Interface:
+    def add_subif(self, interface: str, subif: int) -> _InterfaceT:
         name = self._subif_name(interface, subif)
         for target_port in self.interfaces:
             if target_port.name == name:
@@ -288,7 +343,7 @@ class NetboxDevice(Entity):
         self.interfaces.append(target_port)
         return target_port
 
-    def find_interface(self, name: str) -> Optional[Interface]:
+    def find_interface(self, name: str) -> Optional[_InterfaceT]:
         for iface in self.interfaces:
             if iface.name == name:
                 return iface
