@@ -6,9 +6,10 @@ import sys
 import time
 import warnings
 from collections import OrderedDict as odict
-from itertools import groupby, chain
+from itertools import chain, groupby
 from operator import itemgetter
 from typing import (
+    Any,
     Dict,
     Generator,
     Iterable,
@@ -16,38 +17,37 @@ from typing import (
     Mapping,
     Set,
     Tuple,
-    Union, cast,
+    Union,
+    cast,
 )
 
 import colorama
+from contextlog import get_logger
+
 import annet.deploy
 import annet.deploy_ui
 import annet.lib
-from annet.annlib import jsontools
-from annet.annlib.netdev.views.hardware import HardwareView
-from annet.annlib.rbparser.platform import VENDOR_REVERSES
-from annet.annlib.types import GeneratorType
-from contextlog import get_logger
-
-from annet.deploy import Fetcher, DeployDriver
 from annet import cli_args
 from annet import diff as ann_diff
 from annet import filtering
 from annet import gen as ann_gen
-from annet import patching, rulebook, tabparser, tracing
+from annet import patching, rulebook, tracing
+from annet.vendors import tabparser
+from annet.annlib import jsontools
+from annet.annlib.netdev.views.hardware import HardwareView
+from annet.annlib.types import GeneratorType
+from annet.deploy import DeployDriver, Fetcher
 from annet.diff import file_differ_connector
-from annet.rulebook import deploying
 from annet.filtering import Filterer
 from annet.hardware import hardware_connector
-from annet.output import (
-    format_file_diff,
-    output_driver_connector,
-    print_err_label,
-)
+from annet.output import format_file_diff, output_driver_connector, print_err_label
 from annet.parallel import Parallel, TaskResult
 from annet.reference import RefTracker
+from annet.rulebook import deploying
 from annet.storage import Device, get_storage
-from annet.types import Diff, ExitCode, OldNewResult, Op, PCDiff, PCDiffFile
+from annet.types import Diff, ExitCode, OldNewResult, Op, PCDiff
+from annet.vendors import registry_connector
+
 
 DEFAULT_INDENT = "  "
 
@@ -96,21 +96,45 @@ def _read_old_new_diff_patch(old: Dict[str, Dict], new: Dict[str, Dict], hw: Har
     return rb, diff_obj, pre, patchtree
 
 
-def _read_old_new_hw(old_path: str, new_path: str, args: cli_args.FileInputOptions):
+def _read_old_new_configs(old_path, new_path):
     _logger = get_logger()
+    with open(old_path) as f:
+        _logger.debug("Reading %r ...", old_path)
+        old_config = f.read()
+
+    with open(new_path) as f:
+        _logger.debug("Reading %r ...", new_path)
+        new_config = f.read()
+
+    return old_config, new_config
+
+
+def _read_old_new_hw(old_path: str, old_config: str, new_path: str, new_config: str, args: cli_args.FileInputOptions):
+    _logger = get_logger()
+
     hw = args.hw
     if isinstance(args.hw, str):
         hw = HardwareView(args.hw, "")
 
-    old, old_hw, old_score = _read_device_config(old_path, hw)
-    new, new_hw, new_score = _read_device_config(new_path, hw)
+    try:
+        old, old_hw, old_score = _parse_device_config(old_config, hw)
+    except tabparser.ParserError:
+        _logger.exception("Parser error: %r", old_path)
+        raise
+
+    try:
+        new, new_hw, new_score = _parse_device_config(new_config, hw)
+    except tabparser.ParserError:
+        _logger.exception("Parser error: %r", new_path)
+        raise
+
     hw = new_hw
     if old_score > new_score:
         hw = old_hw
     if old_hw != new_hw:
-        _logger.debug("Old and new detected hw differs, assume %r", hw)
-    dest_name = os.path.basename(new_path)
-    return dest_name, old, new, hw
+        _logger.warning("Old and new detected hw differs, assume %r", hw)
+
+    return old, new, hw
 
 
 @tracing.function
@@ -134,29 +158,24 @@ def _read_old_new_cfgdumps(args: cli_args.FileInputOptions):
         yield (old_path_name, new_path_name)
 
 
-def _read_device_config(path, hw):
-    _logger = get_logger()
-    _logger.debug("Reading %r ...", path)
+def _parse_device_config(text, hw):
     score = 1
+    vendor_registry = registry_connector.get()
 
-    with open(path) as cfgdump_file:
-        text = cfgdump_file.read()
-    try:
-        if not hw:
-            hw, score = guess_hw(text)
-        config = tabparser.parse_to_tree(
-            text=text,
-            splitter=tabparser.make_formatter(hw).split,
-        )
-        return config, hw, score
-    except tabparser.ParserError:
-        _logger.exception("Parser error: %r", path)
-        raise
+    if not hw:
+        hw, score = guess_hw(text)
+
+    config = tabparser.parse_to_tree(
+        text=text,
+        splitter=vendor_registry.match(hw).make_formatter().split,
+    )
+
+    return config, hw, score
 
 
 # =====
 def _format_patch_blocks(patch_tree, hw, indent):
-    formatter = tabparser.make_formatter(hw, indent=indent)
+    formatter = registry_connector.get().match(hw).make_formatter(indent=indent)
     return formatter.patch(patch_tree)
 
 
@@ -170,8 +189,6 @@ def _print_pre_as_diff(pre, show_rules, indent, file=None, _level=0):
             (Op.AFFECTED, colorama.Fore.CYAN + " "),
         ]:
             items = content["items"].items()
-            if not content["attrs"]["multiline"]:
-                items = sorted(items, key=itemgetter(0))
             for (_, diff) in items:  # pylint: disable=redefined-outer-name
                 if show_rules and not rule_printed and not raw_rule == "__MULTILINE_BODY__":
                     print("%s%s# %s%s%s" % (colorama.Style.BRIGHT, colorama.Fore.BLACK, (indent * _level),
@@ -312,16 +329,18 @@ def res_diff_patch(
 def diff(
     args: cli_args.DiffOptions,
     loader: ann_gen.Loader,
-) -> Mapping[Device, Union[Diff, PCDiff]]:
+    device_ids: List[Any]
+) -> tuple[Mapping[Device, Union[Diff, PCDiff]], Mapping[Device, Exception]]:
     """ Сгенерировать конфиг для устройств """
     stdin = args.stdin(filter_acl=args.filter_acl, config=None)
 
     filterer = filtering.filterer_connector.get()
     pool = Parallel(ann_diff.worker, args, stdin, loader, filterer).tune_args(args)
     if args.show_hosts_progress:
-        pool.add_callback(PoolProgressLogger(loader.device_fqdns))
+        fqdns = {k: v for k, v in loader.device_fqdns.items() if k in device_ids}
+        pool.add_callback(PoolProgressLogger(fqdns))
 
-    return pool.run(loader.device_ids, args.tolerate_fails, args.strict_exit_code)
+    return pool.run(device_ids, args.tolerate_fails, args.strict_exit_code)
 
 
 def collapse_texts(texts: Mapping[str, str | Generator[str, None, None]]) -> Mapping[Tuple[str, ...], str]:
@@ -393,9 +412,12 @@ class CliDeployerJob(DeployerJob):
         (diff_obj, patch_tree) = _diff_and_patch(device, old, new, acl_rules,
                                                  res.filter_acl_rules, self.add_comments,
                                                  do_commit=not self.args.dont_commit)
-        cmds = tabparser.make_formatter(device.hw, indent="").cmd_paths(patch_tree)
+
+        formatter = registry_connector.get().match(device.hw).make_formatter(indent="")
+        cmds = formatter.cmd_paths(patch_tree)
         if not cmds:
             return
+
         self._has_diff = True
         self.diffs[device] = diff_obj
         self.cmd_lines.extend(["= %s " % device.hostname, ""])
@@ -500,7 +522,6 @@ class Deployer:
 
         self._collapseable_diffs = {}
         self._diff_lines: List[str] = []
-        self._filterer = filtering.filterer_connector.get()
 
     def parse_result(self, job: DeployerJob, result: ann_gen.OldNewResult):
         logger = get_logger(job.device.hostname)
@@ -549,7 +570,7 @@ class Deployer:
                 dest_name = "= %s" % ", ".join([dev.hostname for dev in devices])
                 diff_lines.extend([dest_name, ""])
 
-            for line in tabparser.make_formatter(devices[0].hw).diff(diff_obj):
+            for line in registry_connector.get().match(devices[0].hw).make_formatter().diff(diff_obj):
                 diff_lines.append(line)
             diff_lines.append("")
         return diff_lines
@@ -581,29 +602,44 @@ class Deployer:
         return ans
 
     def check_diff(self, result: annet.deploy.DeployResult, loader: ann_gen.Loader):
-        success_device_ids = []
-        for host, hres in result.results.items():
-            device = self.fqdn_to_device[host]
-            if (
-                not isinstance(hres, Exception) and
-                host not in self.empty_diff_hostnames and
-                device.is_pc()
-            ):
-                success_device_ids.append(device.id)
         diff_args = self.args.copy_from(
             self.args,
             config="running",
         )
-        if diff_args.query:
-            ann_gen.live_configs = None
-            diffs = diff(diff_args, loader, success_device_ids, self._filterer)
-            non_pc_diffs = {dev: diff for dev, diff in diffs.items() if not isinstance(diff, PCDiff)}
-            devices_to_diff = ann_diff.collapse_diffs(non_pc_diffs)
-            devices_to_diff.update({(dev,): diff for dev, diff in diffs.items() if isinstance(diff, PCDiff)})
-        else:
-            devices_to_diff = {}
-        for devices, diff_obj in devices_to_diff.items():
-            if diff_obj:
+        if not diff_args.query:
+            return
+
+        # clear cache
+        ann_gen.live_configs = None
+
+        # collect new diffs for devices on which we had successfully uploaded something
+        success_device_ids = []
+        for host, hres in result.results.items():
+            if (
+                not isinstance(hres, Exception) and
+                host not in self.empty_diff_hostnames
+            ):
+                device = self.fqdn_to_device[host]
+                success_device_ids.append(device.id)
+        diffs, failed = diff(diff_args, loader, success_device_ids)
+        for device_id, exc in failed.items():
+            self.failed_configs[loader.get_device(device_id).fqdn] = exc
+
+        # "collapse" non-PC diffs
+        diffs_by_device_id = ann_diff.collapse_diffs({
+            loader.get_device(device_id): diff
+            for device_id, diff in diffs.items()
+            if diff and not isinstance(diff, PCDiff)
+        })
+        # add PC diffs as is
+        diffs_by_device_id.update({
+            (loader.get_device(device_id),): diff
+            for device_id, diff in diffs.items()
+            if diff and isinstance(diff, PCDiff)
+        })
+        if diffs_by_device_id:
+            print("The diff is still present:")
+            for devices, diff_obj in diffs_by_device_id.items():
                 for dev in devices:
                     self.failed_configs[dev.fqdn] = Warning("Deploy OK, but diff still exists")
                 if isinstance(diff_obj, PCDiff):
@@ -734,12 +770,15 @@ def file_diff_worker(
             if diff_text:
                 yield diff_file.label, diff_text, False
     else:
-        dest_name, old, new, hw = _read_old_new_hw(old_path, new_path, args)
+        old_config, new_config = _read_old_new_configs(old_path, new_path)
+        if old_config == new_config:
+            return
+
+        old, new, hw = _read_old_new_hw(old_path, old_config, new_path, new_config, args)
         _, __, pre, ___ = _read_old_new_diff_patch(old, new, hw, add_comments=False)
-        diff_lines = ann_diff.gen_pre_as_diff(pre, args.show_rules, args.indent, args.no_color)
-        diff_text = "".join(diff_lines)
-        if diff_text:
-            yield dest_name, diff_text, False
+
+        if diff_lines := ann_diff.gen_pre_as_diff(pre, args.show_rules, args.indent, args.no_color):
+            yield os.path.basename(new_path), "".join(diff_lines), False
 
 
 @tracing.function
@@ -751,18 +790,23 @@ def file_patch(args: cli_args.FilePatchOptions):
 
 
 def file_patch_worker(old_new: Tuple[str, str], args: cli_args.FileDiffOptions) -> Generator[
-    Tuple[str, str, bool], None, None]:
+    Tuple[str, str, bool], None, None
+]:
     old_path, new_path = old_new
     if os.path.isdir(old_path) and os.path.isdir(new_path):
         for relative_cfg_path, cfg_text in ann_gen.load_pc_config(new_path).items():
             label = os.path.join(os.path.basename(new_path), relative_cfg_path)
             yield label, cfg_text, False
     else:
-        dest_name, old, new, hw = _read_old_new_hw(old_path, new_path, args)
+        old_config, new_config = _read_old_new_configs(old_path, new_path)
+        if old_config == new_config:
+            return
+
+        old, new, hw = _read_old_new_hw(old_path, old_config, new_path, new_config, args)
         _, __, ___, patch_tree = _read_old_new_diff_patch(old, new, hw, args.add_comments)
         patch_text = _format_patch_blocks(patch_tree, hw, args.indent)
         if patch_text:
-            yield dest_name, patch_text, False
+            yield os.path.basename(new_path), patch_text, False
 
 
 def guess_hw(config_text: str):
@@ -770,14 +814,17 @@ def guess_hw(config_text: str):
     текста конфига и annet/rulebook/texts/*.rul"""
     scores = {}
     hw_provider = hardware_connector.get()
-    for vendor in VENDOR_REVERSES:
+    vendor_registry = registry_connector.get()
+    for vendor in vendor_registry:
         hw = hw_provider.vendor_to_hw(vendor)
-        fmtr = tabparser.make_formatter(hw)
         rb = rulebook.get_rulebook(hw)
+        fmtr = vendor_registry[vendor].make_formatter()
+
         try:
             config = tabparser.parse_to_tree(config_text, fmtr.split)
         except Exception:
             continue
+
         pre = patching.make_pre(patching.make_diff({}, config, rb, []))
         metric = _count_pre_score(pre)
         scores[metric] = hw
