@@ -1,6 +1,6 @@
 from collections.abc import Sequence, Iterable
 from dataclasses import dataclass, field
-from typing import Literal, Union, Optional
+from typing import Literal, Union, Optional, Annotated
 
 
 class VidRange:
@@ -139,7 +139,15 @@ class BFDTimers:
     multiplier: int = 4
 
 
-Family = Literal["ipv4_unicast", "ipv6_unicast", "ipv4_labeled_unicast", "ipv6_labeled_unicast", "l2vpn_evpn"]
+Family = Literal[
+    "ipv4_unicast",
+    "ipv6_unicast",
+    "ipv4_vpn_unicast",
+    "ipv6_vpn_unicast",
+    "ipv4_labeled_unicast",
+    "ipv6_labeled_unicast",
+    "l2vpn_evpn",
+]
 
 
 @dataclass(frozen=True)
@@ -154,7 +162,8 @@ class PeerOptions:
     send_lcommunity: Optional[bool] = None
     send_extcommunity: Optional[bool] = None
     send_labeled: Optional[bool] = None
-    import_limit: Optional[bool] = None
+    import_limit: Optional[int] = None
+    import_limit_action: Optional[str] = None
     teardown_timeout: Optional[bool] = None
     redistribute: Optional[bool] = None
     passive: Optional[bool] = None
@@ -191,6 +200,7 @@ class PeerOptions:
     soft_reconfiguration_inbound: Optional[bool] = None
     not_active: Optional[bool] = None
     mtu: Optional[int] = None
+    password: Optional[str] = None
 
 
 @dataclass
@@ -232,7 +242,9 @@ class FamilyOptions:
     vrf_name: str = ""
     multipath: int = 0
     global_multipath: int = 0
-    aggregate: Aggregate = field(default_factory=Aggregate)
+    aggregate: Aggregate = field(default_factory=Aggregate)  # use `aggregates` instead
+    aggregates: tuple[Aggregate, ...] = ()
+    af_loops: Optional[int] = None
     redistributes: tuple[Redistribute, ...] = ()
     allow_default: bool = False
     aspath_relax: bool = False
@@ -267,7 +279,8 @@ class PeerGroup:
     send_lcommunity: bool = False
     send_extcommunity: bool = False
     send_labeled: bool = False
-    import_limit: bool = False
+    import_limit: int = False
+    import_limit_action: Optional[str] = None
     teardown_timeout: bool = False
     redistribute: bool = False
     passive: bool = False
@@ -304,6 +317,7 @@ class PeerGroup:
     soft_reconfiguration_inbound: bool = False
     not_active: bool = False
     mtu: int = 0
+    password: Optional[str] = None
 
 
 @dataclass
