@@ -3,19 +3,10 @@ from __future__ import annotations
 import pkgutil
 import re
 import types
-from typing import (
-    FrozenSet,
-    Iterable,
-    List,
-    Optional,
-    Set,
-    Union,
-)
+from typing import FrozenSet, Iterable, List, Optional, Set, Union
+
+from annet.lib import flatten, mako_render
 
-from annet.lib import (
-    flatten,
-    mako_render,
-)
 from .base import BaseGenerator, _filter_str
 from .exceptions import NotSupportedDevice
 
@@ -24,6 +15,7 @@ class Entire(BaseGenerator):
     TYPE = "ENTIRE"
     TAGS: List[str] = []
     REQUIRED_PACKAGES: FrozenSet[str] = frozenset()
+    ENSURE_END_NEWLINE = False
 
     def __init__(self, storage):
         self.storage = storage
@@ -38,8 +30,7 @@ class Entire(BaseGenerator):
     def run(self, device) -> Union[None, str, Iterable[Union[str, tuple]]]:
         raise NotImplementedError
 
-    def reload(self, device) -> Optional[
-        str]:  # pylint: disable=unused-argument
+    def reload(self, device) -> Optional[str]:  # pylint: disable=unused-argument
         return
 
     def get_reload_cmds(self, device) -> str:
@@ -90,4 +81,7 @@ class Entire(BaseGenerator):
             assert re.search(r"\bNone\b", text) is None, \
                 "Found 'None' in yield result: %s" % text
             parts.append(text)
-        return "\n".join(parts)
+        ret = "\n".join(parts)
+        if self.ENSURE_END_NEWLINE and not ret.endswith("\n"):
+            ret += "\n"
+        return ret
