@@ -1,11 +1,11 @@
 from typing import Annotated, Optional, Union
 
-from annet.bgp_models import Family, Redistribute
+from annet.bgp_models import Family, Redistribute, Aggregate
 from .basemodel import BaseMeshModel, Concat, DictMerge, Merge, KeyDefaultDict
 from .peer_models import MeshPeerGroup
 
 
-class Aggregate(BaseMeshModel):
+class _Aggregate(BaseMeshModel):
     policy: str
     routes: Annotated[tuple[str, ...], Concat()]
     as_path: str
@@ -17,13 +17,15 @@ class Aggregate(BaseMeshModel):
 
 class FamilyOptions(BaseMeshModel):
     def __init__(self, **kwargs):
-        kwargs.setdefault("aggregate", Aggregate())
+        kwargs.setdefault("aggregate", _Aggregate())
         super().__init__(**kwargs)
     family: Family
     vrf_name: str
     multipath: int
     global_multipath: int
-    aggregate: Annotated[Aggregate, Merge()]
+    aggregate: Annotated[_Aggregate, Merge()]  # use `aggregates` instead
+    aggregates: Annotated[tuple[Aggregate, ...], Concat()]
+    af_loops: int
     redistributes: Annotated[tuple[Redistribute, ...], Concat()]
     allow_default: bool
     aspath_relax: bool
@@ -40,12 +42,16 @@ class _FamiliesMixin:
     def __init__(self, **kwargs):
         kwargs.setdefault("ipv4_unicast", FamilyOptions(family="ipv4_unicast"))
         kwargs.setdefault("ipv6_unicast", FamilyOptions(family="ipv6_unicast"))
+        kwargs.setdefault("ipv4_vpn_unicast", FamilyOptions(family="ipv4_vpn_unicast"))
+        kwargs.setdefault("ipv6_vpn_unicast", FamilyOptions(family="ipv6_vpn_unicast"))
         kwargs.setdefault("ipv4_labeled_unicast", FamilyOptions(family="ipv4_labeled_unicast"))
         kwargs.setdefault("ipv6_labeled_unicast", FamilyOptions(family="ipv6_labeled_unicast"))
         kwargs.setdefault("l2vpn_evpn", FamilyOptions(family="l2vpn_evpn"))
         super().__init__(**kwargs)
     ipv4_unicast: Annotated[FamilyOptions, Merge()]
     ipv6_unicast: Annotated[FamilyOptions, Merge()]
+    ipv4_vpn_unicast: Annotated[FamilyOptions, Merge()]
+    ipv6_vpn_unicast: Annotated[FamilyOptions, Merge()]
     ipv4_labeled_unicast: Annotated[FamilyOptions, Merge()]
     ipv6_labeled_unicast: Annotated[FamilyOptions, Merge()]
     l2vpn_evpn: Annotated[FamilyOptions, Merge()]
@@ -55,6 +61,8 @@ class VrfOptions(_FamiliesMixin, BaseMeshModel):
     def __init__(self, vrf_name: str, **kwargs):
         kwargs.setdefault("ipv4_unicast", FamilyOptions(family="ipv4_unicast", vrf_name=vrf_name))
         kwargs.setdefault("ipv6_unicast", FamilyOptions(family="ipv6_unicast", vrf_name=vrf_name))
+        kwargs.setdefault("ipv4_vpn_unicast", FamilyOptions(family="ipv4_unicast", vrf_name=vrf_name))
+        kwargs.setdefault("ipv6_vpn_unicast", FamilyOptions(family="ipv6_unicast", vrf_name=vrf_name))
         kwargs.setdefault("ipv4_labeled_unicast", FamilyOptions(family="ipv4_labeled_unicast", vrf_name=vrf_name))
         kwargs.setdefault("ipv6_labeled_unicast", FamilyOptions(family="ipv6_labeled_unicast", vrf_name=vrf_name))
         kwargs.setdefault("l2vpn_evpn", FamilyOptions(family="l2vpn_evpn", vrf_name=vrf_name))
