@@ -22,11 +22,34 @@ from .registry import (
 logger = getLogger(__name__)
 
 
+@dataclass(frozen=True)
+class TargetInterface:
+    subif: int | None = None
+    svi: int | None = None
+    lag: int | None = None
+    port: str | None = None
+
+
 @dataclass(frozen=True)
 class PeerKey:
     fqdn: str
     addr: str
     vrf: str
+    interface: TargetInterface
+
+
+def target_interface(
+        peer: DirectPeerDTO | IndirectPeerDTO | VirtualLocalDTO,
+        ports: list[str],
+) -> TargetInterface:
+    subif = getattr(peer, "subif", None)
+    svi = getattr(peer, "svi", None)
+    lag = getattr(peer, "lag", None)
+    port = None
+    if not (svi or lag):
+        if len(ports) == 1:
+            port = ports[0]
+    return TargetInterface(subif, svi, lag, port)
 
 
 class Pair(BaseMeshModel):
@@ -171,7 +194,8 @@ class MeshExecutor:
                 peer_key = PeerKey(
                     fqdn=pair.device.fqdn,
                     addr=addr,
-                    vrf=getattr(pair.connected, "vrf", "")
+                    vrf=getattr(pair.connected, "vrf", ""),
+                    interface=target_interface(pair.local, [p[0] for p in ports]),
                 )
                 try:
                     if peer_key in neighbor_peers:
@@ -194,7 +218,7 @@ class MeshExecutor:
         for rule in self._registry.lookup_virtual(device.fqdn):
             for order_number in rule.num:
                 handler_name = self._handler_name(rule.handler)
-                logger.debug("Running direct handler: %s", handler_name)
+                logger.debug("Running virtual handler: %s", handler_name)
                 session = MeshSession()
                 peer_device = VirtualLocal(rule.match, device)
                 peer_virtual = VirtualPeer(num=order_number)
@@ -219,11 +243,6 @@ class MeshExecutor:
                         f"peer data for device `{device.fqdn}`:\n" + str(e)
                     ) from e
 
-                if not hasattr(device_dto, "svi"):
-                    raise ValueError(
-                        f"Handler `{handler_name}` did not provide `svi` number. "
-                        "Virtual peer must be connected to SVI interface."
-                    )
                 pair = VirtualPair(local=device_dto, connected=virtual_dto)
                 virtual_peers.append(pair)
         return virtual_peers
@@ -283,7 +302,8 @@ class MeshExecutor:
             peer_key = PeerKey(
                 fqdn=connected_device.fqdn,
                 addr=addr,
-                vrf=getattr(connected_dto, "vrf", "")
+                vrf=getattr(connected_dto, "vrf", ""),
+                interface=target_interface(pair.local, []),
             )
             try:
                 if peer_key in connected_peers:
@@ -345,11 +365,11 @@ class MeshExecutor:
         target_interface.add_addr(changes.addr, changes.vrf)
         return target_interface.name
 
-    def _apply_indirect_interface_changes(
-            self, device: Device, neighbor: Device, ifname: Optional[str], changes: InterfaceChanges,
+    def _apply_nondirect_interface_changes(
+            self, device: Device, ifname: Optional[str], changes: InterfaceChanges,
     ) -> Optional[str]:
         if changes.lag is not None:
-            raise ValueError("LAG creation unsupported for indirect peers")
+            raise ValueError("LAG creation unsupported for indirect and virtual peers")
         elif changes.subif is not None:
             target_interface = device.add_subif(ifname, changes.subif)
         elif changes.svi is not None:
@@ -363,9 +383,6 @@ class MeshExecutor:
         target_interface.add_addr(changes.addr, changes.vrf)
         return target_interface.name
 
-    def _apply_virtual_interface_changes(self, device: Device, local: VirtualLocalDTO) -> str:
-        return device.add_svi(local.svi).name  # we check if SVI configured in execute method
-
     def execute_for(self, device: Device) -> BgpConfig:
         all_fqdns = self._storage.resolve_all_fdnds()
 
@@ -378,23 +395,23 @@ class MeshExecutor:
                 device,
                 direct_pair.device,
                 direct_pair.ports,
-                to_interface_changes(direct_pair.local),
+                to_interface_changes(direct_pair.local, direct_pair.connected),
             )
             peers.append(self._to_bgp_peer(direct_pair, target_interface))
 
         for virtual_pair in self._execute_virtual(device):
-            target_interface = self._apply_virtual_interface_changes(
+            target_interface = self._apply_nondirect_interface_changes(
                 device,
-                virtual_pair.local,
+                getattr(virtual_pair.local, "ifname", None),
+                to_interface_changes(virtual_pair.local, virtual_pair.connected),
             )
             peers.append(self._virtual_to_bgp_peer(virtual_pair, target_interface))
 
         for connected_pair in self._execute_indirect(device, all_fqdns):
-            target_interface = self._apply_indirect_interface_changes(
+            target_interface = self._apply_nondirect_interface_changes(
                 device,
-                connected_pair.device,
                 getattr(connected_pair.local, "ifname", None),
-                to_interface_changes(connected_pair.local),
+                to_interface_changes(connected_pair.local, connected_pair.connected),
             )
             peers.append(self._to_bgp_peer(connected_pair, target_interface))
 
