@@ -84,7 +84,7 @@ class Checkable:
         self.interface = ConditionFactory[str](MatchField.interface, ["=="])
         self.protocol = ConditionFactory[str](MatchField.protocol, ["=="])
         self.net_len = ConditionFactory[int](MatchField.net_len, ["==", "!="])
-        self.local_pref = ConditionFactory[int](MatchField.local_pref, ["<"])
+        self.local_pref = ConditionFactory[int](MatchField.local_pref, ["<", "==", ">=", "<=", "BETWEEN_INCLUDED"])
         self.metric = ConditionFactory[int](MatchField.metric, ["=="])
         self.family = ConditionFactory[int](MatchField.family, ["=="])
         self.as_path_length = ConditionFactory[int](MatchField.as_path_length, ["==", ">=", "<=", "BETWEEN_INCLUDED"])
